library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity byteport is
  port (
    clear    : in  std_logic;
    clk      : in  std_logic;
    ibus     : in  std_logic_vector (7 downto 0);
    obus     : out std_logic_vector (7 downto 0);
    loadport : in  std_logic;
    loadddr  : in  std_logic;
    readddr  : in  std_logic;
    portdata : out std_logic_vector (7 downto 0)
    );
end byteport;

architecture behavioral of byteport is

  signal outreg    : std_logic_vector (7 downto 0);
  signal ddrreg    : std_logic_vector (7 downto 0);
  signal loadport1 : std_logic;
  signal loadport2 : std_logic;
  signal loadddr1  : std_logic;
  signal loadddr2  : std_logic;
  signal tsoutreg  : std_logic_vector (7 downto 0);

begin
  awordioport : process (
    clk, ddrreg, outreg, readddr, tsoutreg)
  begin
    if clk'event and clk = '1' then
      loadport1 <= loadport;
      loadport2 <= loadport1;
      loadddr1  <= loadddr;
      loadddr2  <= loadddr1;
      if loadport1 = '0' and loadport2 = '1' then
        outreg <= ibus;
      end if;
      if loadddr1 = '0' and loadddr2 = '1' then
        ddrreg <= ibus;
      end if;
      if clear = '1' then ddrreg <= x"00"; end if;
    end if;  -- clk

    for i in 0 to 7 loop
      if ddrreg(i) = '1' then
        tsoutreg(i) <= outreg(i);
      else
        tsoutreg(i) <= 'Z';
      end if;
    end loop;

    portdata <= tsoutreg;

    if readddr = '1' then
      obus <= ddrreg;
    else
      obus <= "ZZZZZZZZ";
    end if;

  end process;
end behavioral;
