type LongIntRec = record
  LowWord  : word;
  HighWord : word;
end;

type WordRec = record
  LowByte : byte;
  HighByte : byte;
end;

var
TheComPort : word;
TheBaudRate : real;
BusIntfc : boolean;
Commandport : word;
Dataport : word;
RomAddPort : word;
ROMDataPort : word;

const
TwoToThe31 = 2147483647;
WriteFlag : word = $8000;
AxisOffset = 11;
LongFlag : word = $400;
CommandPortOffset = 2;
DataPortOffset = 4;
RomAddOffset =  8;
RomDataOffset  = 10;
ROMSize1 = 1024;
ROMSize2 = 2048;
SignatureOffset = -1;
Signature = $CAFE;


BusyMask : word = $8000;
Sysclk = 50E6; { nominally 50 MHZ for -5 parts, -6 can run at 66 MHZ }
CPUCycleTime = Sysclk;

type

Event = record
  ER_condptr : word;
  ER_condxor : word;
  ER_condand_ptr : word;
  ER_condflag  : word;
  ER_operptr : word;
  ER_operxor : word;
  ER_operand : word;
  ER_operor_src  : word;
end;

{ These are are machine created parameter location (smotcons) and }
{ bit mask (smotbits) include files... do not edit these files directly }
{ they are created from the source files with MAKEINC }

const
{$I SMOTPARM.PAS}
{$I SMOTBITS.PAS}

procedure BusWriteParam(command: word;thedata : longint);
{ full 32 bit parameter write - 2 words }
var foo : word;
begin
  portw[DataPort] := word(thedata);
  portw[DataPort+2] := word(thedata shr 16);
  portw[CommandPort] := command;
  repeat
    foo := portw[CommandPort];
  until (foo and BusyMask = 0);
end;

procedure BusWriteParamWord(command,thedata : word);
{16 bit only parameter write}
var foo : word;
begin
  portw[DataPort] := word(thedata);
  portw[CommandPort] := command;
  repeat
    foo := portw[CommandPort];
  until (foo and BusyMask = 0);
end;

function BusReadParam(command: word):longint;
{full 32 bit parameter read }
var
foo : word;
rdata : longint;
begin
  portw[CommandPort] := command;
  repeat
    foo := portw[CommandPort];
  until (foo and BusyMask = 0);
  rdata := Portw[DataPort] + (portw[DataPort+2] * 65536);
  BusReadParam :=rdata;
end;

function BusReadParamWord(command: word):word;
{ 16 bit parameter read }
var
foo : word;
rdata : longint;
begin
  portw[CommandPort] := command;
  repeat
    foo := portw[CommandPort];
  until (foo and BusyMask = 0);
  rdata := Portw[DataPort];
  BusReadParamWord :=rdata;
end;

function BusReadStatus:word;
{ read back status reg }
begin
  BusReadStatus := portw[CommandPort];
end;


{$I serhost.pas}

procedure WriteParam(theaxis: byte;theparm: word;thedata : longint);
var command : word;
{ full 32 bit parameter write - 2 words }
begin
  command := (theparm or WriteFlag or LongFlag or (theaxis shl AxisOffset));
  if BusIntfc then
    BusWriteParam(command,thedata)
  else
    SerWriteParam(TheComport,command,thedata);
end;

procedure WriteParamWord(theaxis: byte;theparm: word;thedata : word);
var command : word;
{ 16 bit parameter write }
begin
  command := (theparm or WriteFlag or (theaxis shl AxisOffset));
  if BusIntfc then
    BusWriteParamWord(command,thedata)
  else
    SerWriteParamWord(TheComport,command,thedata);
end;

function ReadParam(theaxis: byte;theparm: word):longint;
var command : word;
{full 32 bit parameter read }
begin
  command := (theparm or LongFlag or (theaxis shl AxisOffset));
  if BusIntfc then
    ReadParam := BusReadParam(command)
  else
    ReadParam := SerReadParam(TheComport,command);
end;

function ReadParamWord(theaxis: byte;theparm: word):word;
var command : word;
{16 bit parameter read }
begin
  command := (theparm or (theaxis shl AxisOffset));
  if BusIntfc then
    ReadParamWord := BusReadParamWord(command)
  else
    ReadParamWord := SerReadParamWord(TheComport,command);
end;

procedure Move(axis : word;nextpos : longint; acc : longint; maxv : longint);
begin
  WriteParamword(axis,MaxNegErrLoc,0);
  WriteParamword(axis,MaxPosErrLoc,0);
  WriteParam(axis,NextPosLoc,nextpos);
  WriteParam(axis,AccelLoc,acc);
  WriteParam(axis,SlewLimitLoc,maxv);
  WriteParamWord(axis,GoLoc,TrueF);
end;

procedure WaitforMoveDone(axis : word);
var
temp : word;
begin
  repeat
    temp := ReadParamWord(axis,GoLoc);
  until (temp = 0);
end;

procedure WaitTillStopped(axis : word);
var
temp : word;
begin
  repeat
    temp := ReadParamWord(axis,MotionLoc);
  until (temp = 0);
end;

procedure WaitforMoveDoneWithStats(axis : word);
var
temp : word;
cyctime,maxcyc : word;
begin
  { wait for motors to be done and sample loop time statistics}
  maxcyc := 0;
  repeat
    cyctime := ReadParamWord(0,ProcTimerLoc);
    if (cyctime > maxcyc) then maxcyc := cyctime;
    temp := ReadParamWord(axis,GoLoc);
  until ((temp and $00FF) = 0);  { check low byte for done-a-tude }
  writeln('Maximum cycle time = ',maxcyc*1e6*CPUCycleTime:4:0,'uS');
  writeln('MaxNegErr = ',Integer(ReadParamWord(axis,MaxNegErrLoc)),' MaxPosErr = ',ReadParamWord(axis,MaxPosErrLoc));
end;

procedure ResetAxis(axis : byte);
begin
  WriteParamWord(axis,ResetLoc,FalseF);
  while ReadParamWord(axis,ResetLoc) = 0 do;
end;

procedure SetHomePos(axis:word);
begin
  WriteParamWord(axis,PIDLoc,FalseF); { turn off PID loop whilst futzing }
  WriteParamWord(axis,GoLoc,FalseF);    { turn off profile whilst futzing }
  WriteParamWord(axis,CntCntlPLoc,CCFilterBit or CCClrBit); {quad filter and clear}
  WriteParam(axis,DesPosLoc,0);  { clear Desired Pos reg }
  WriteParam(axis,NextPosLoc,0); { clear Next Pos reg }
  WriteParam(axis,LastPosLoc,0); { clear Last Pos reg }
  WriteParamWord(axis,ErrorLoc,0); { clear any expe }
  WriteParamWord(axis,PIDLoc,TrueF); { re-enable PID }
end;

procedure SeekHomePos(axis:word;nextpos : longint; acc : longint; maxv : longint);
begin
  WriteParamWord(axis,CntCntlPLoc,CCFilterBit or CCClrOnIDXBit); {quad filter and clear-on-index}
  { You need to set the CCIdxPol bit if you want rising edge }
  { index detection instead of falling edge detection }
  WriteParamWord(axis,ErrorLoc,0); { clear any expe }
  WriteParamWord(axis,ErrorLoc,50000); { set error limit high }
  WriteParamWord(axis,HomeLoc,TrueF); { set the home flag }
  Move(axis,nextpos,acc,maxv);
  WaitForMoveDone(axis);
  WriteParamWord(axis,HomeLoc,FalseF); { now that we're done, reset the home flag }
end;


procedure Stop(axis:word);
begin
  WriteParamWord(axis,GoLoc,FalseF);
  WriteParam(axis,desvelloc,0);
  WaitTillStopped(axis);
end;

procedure QuickStop(axis:word;acc :longint);
begin
  WriteParam(axis,AccelLoc,acc);  { set accel up to stop quick }
  Stop(Axis);
end;

procedure RecoverFromExpe(axis:word);
var
realpos : longint;
begin
  if ReadParamWord(axis,ErrorLoc) <> 0 then
  begin
    writeln('Axis ',Axis,' stopped due to position error > ',ReadParam(Axis,MaxPosErrLoc));
    RealPos := ReadParam(Axis,EncPLoc); { get current actual position }
    writeln('Setting DesPos = Real Position: ',RealPos:12);
    WriteParam(Axis,DesPosLoc,RealPos);  { store it into desired Pos reg }
    WriteParam(Axis,LastPosLoc,RealPos); { store it into Last Pos reg so we dont  }
                                         { so we dont get a large KD generated thunk }
                                         { when we re-enable the PID loop }
    WriteParam(Axis,IntegralLoc,0);      { clear accumulated error (anti-thunk) }
    writeln('Clearing error and re-enabling PID on stopped channel');
    WriteParamWord(Axis,ErrorLoc,0);
    WriteParamWord(Axis,PIDLoc,1);
  end;
end;

function ReadRealPosition(Axis:byte): longint;
begin
  ReadRealPosition := ReadParam(axis,EncPLoc);
end;

function ReadFixup(Axis:byte): longint;
begin
  ReadFixup := ReadParam(axis,FixupLoc);
end;

function ReadDesVelocity(Axis:byte): longint;
begin
  ReadDesVelocity := ReadParam(axis,DesVelLoc);
end;

function ReadVelocity(Axis:byte): longint;
begin
  ReadVelocity := ReadParam(axis,VelocityLoc);
end;

function ReadDesPosition(Axis:byte): longint;
begin
  ReadDesPosition := ReadParam(axis,DesPosLoc);
end;

function ReadRealPositionShort(Axis:byte): integer;
begin
  ReadRealPositionShort := ReadParamWord(axis,EncPLoc);
end;

function ReadDesPositionShort(Axis:byte): integer;
begin
  ReadDesPositionShort := ReadParamWord(axis,DesPosLoc);
end;

function ReadDrive(Axis:byte): integer;
begin
  ReadDrive := ReadParamWord(axis,PWMLoc);
end;

function ReadNegError(axis : byte) : integer;
begin
  ReadNegError := Integer(ReadParamWord(axis,MaxNegErrLoc));
end;

function ReadPosError(axis : byte) : word;
begin
  ReadPosError := (ReadParamWord(axis,MaxPosErrLoc));
end;

procedure ClearPosErrs(axis: byte);
begin
  WriteParamWord(axis,MaxPosErrLoc,0);
  WriteParamWord(axis,MaxNegErrLoc,0);
end;

procedure LoadKP(Axis:byte;kp:word);
begin
  WriteParamWord(axis,KPLOC,kp);
end;

procedure LoadKD(Axis:byte;kd:word);
begin
  WriteParamWord(axis,KDLOC,kd);
end;

procedure LoadKDFIL(Axis:byte;kdfil:word);
begin
  WriteParamWord(axis,KDFILLOC,kdfil);
end;

procedure LoadKK(Axis:byte;kk:word);
begin
  WriteParamWord(axis,KKLOC,kk);
end;

procedure LoadKFF(Axis:byte;kff:word);
begin
  WriteParamWord(axis,KFFLOC,kff);
end;

procedure LoadKI(Axis:byte;ki:longint);
begin
  WriteParam(axis,KILOC,ki);
end;

procedure LoadKIL(Axis:byte;kil:word);
begin
  WriteParamWord(axis,KILLOC,kil);
end;

procedure LoadKF1(Axis:byte;kf1:word);
begin
  WriteParamWord(axis,KF1LOC,kf1);
end;

procedure LoadKF2(Axis:byte;kf2:word);
begin
  WriteParamWord(axis,KF2LOC,kf2);
end;

procedure LoadPostScale(sint : word);
begin
  WriteParamWord(0,PostScaleLoc,sint);
end;

procedure LoadPreScale(sint : word);
begin
  WriteParamWord(0,PreScaleLoc,sint);
end;

procedure LoadPosition(axis:byte;pos:longint);
begin
  WriteParam(axis,NextPosLoc,pos);
end;

procedure LoadVelocity(axis:byte;velo:longint);
begin
  WriteParam(axis,SlewLimitLoc,velo);
end;

procedure LoadAcceleration(axis:byte;accel:longint);
begin
  WriteParam(axis,AccelLoc,accel);
end;

procedure ProfileOff(axis: byte);
begin
  WriteParamWord(axis,ProfileLoc,FalseF);
end;

procedure ProfileOn(axis: byte);
begin
  WriteParamWord(axis,ProfileLoc,Truef);
end;

procedure PIDOff(axis : byte);
begin
  WriteParamWord(axis,PIDLoc,FalseF);
end;

procedure PIDOn(axis : byte);
begin
  WriteParamWord(axis,PIDLoc,Truef);
end;

function ProfileOnQ(axis : byte): boolean;
begin
  if ReadParamWord(axis,ProFileLoc) <> 0 then ProfileOnQ := true else ProfileOnQ := false;
end;

procedure TurnOffMotor(axis:byte);
begin
  WriteParamWord(axis,EnaLoc,FalseF);
  WriteParamWord(axis,PIDLoc,FalseF);
  WriteParamWord(axis,ProfileLoc,FalseF);
end;

procedure TurnOnMotor(axis:byte);
begin
  WriteParamWord(axis,EnaLoc,TrueF);
  WriteParamWord(axis,PIDLoc,TrueF);
  WriteParamWord(axis,ProfileLoc,TrueF);
end;

procedure LoadDirectionFlag(axis:byte;flag:word);
begin
  WriteParamWord(axis,DirInvloc,flag);
end;

procedure LoadProctimer(val:word);
begin
  WriteParamWord(0,ProcTimerLoc,val);
end;

function ReadProctimer :word;
begin
  ReadProcTimer := ReadParamWord(0,ProcTimerLoc);
end;

procedure LoadTimeout(val:word);
begin
  WriteParamWord(0,TimeOutLoc,val);
end;

function ReadTimeout :word;
begin
  ReadTimeOut := ReadParamWord(0,TimeOutLoc);
end;

procedure StartTrajectory(axis:byte);
begin
  WriteParamWord(axis,GoLoc,TrueF);
end;

procedure ResetMot;
begin
  TurnOffMotor(0);
  TurnOffMotor(1);
  TurnOffMotor(2);
  TurnOffMotor(3);
end;

procedure LoadPosErrForStop(axis: byte;pos:longint);
begin
  WriteParamWord(axis,MaxPosErrLoc,pos);
end;

function ReadMinorSWVersion: byte;
begin
  ReadMinorSWVersion := lo(ReadParamWord(0,swrevisionloc));
end;

function ReadMajorSWVersion: byte;
begin
  ReadMajorSWVersion := hi(ReadParamWord(0,swrevisionloc));
end;

function ReadHWVersion: word;
begin
  ReadHWVersion := ReadParamWord(0,hwrevisionLoc);
end;

function ReadNaxis: word;
begin
  ReadNaxis := ReadParamWord(0,NaxisLoc);
end;


function HardwareThere : boolean;
begin
  HardWareThere := false;
  if BusIntfc then
  begin
    portw[ROMAddPort] := ROMSize1 + SignatureOffset;
    if portw[ROMDataPort] = Signature then HardwareThere := true;
  end
  else
  begin
    if SerSync(TheComPort) then HardwareThere := true;
  end;
end;

procedure LEDs(Axis : byte; Param : word);
begin
  writeParamWord(0,LEDAxisLoc,Axis);
  writeParamWord(0,LEDLoc,Param);
end;

function AxisEvent(axis,num : byte;evrec : Event) : boolean;
var
eventptr : word;
oldnum : word;
begin
  AxisEvent := false;
  { need to add protection from over allocation }
  oldnum := ReadParamWord(axis,eventsloc);
  if (num < oldnum +2) then { we are re-writing an existing event or adding one }
  begin
    AxisEvent := true;
    eventptr := event1loc + eventblocksize*(num-1);
    WriteParamWord(axis,eventptr+operptr,nullLoc); {disable output part of event until done }
    with evrec do
    begin
      WriteParamWord(axis,eventptr+condptr,ER_condptr);
      WriteParamWord(axis,eventptr+condxor,ER_condxor);
      WriteParamWord(axis,eventptr+condand,ER_condand_ptr);
      WriteParamWord(axis,eventptr+condflag,ER_condflag);
      WriteParamWord(axis,eventptr+operxor,ER_operxor);
      WriteParamWord(axis,eventptr+operand,ER_operand);
      WriteParamWord(axis,eventptr+operor,ER_operor_src);
      WriteParamWord(axis,eventptr+operptr,ER_operptr);  {do last}
    end;
    if num > oldnum then WriteParamWord(axis,eventsloc,num);
  end;
end;

function GlobalEvent(num : byte;evrec : Event) : boolean;
var
eventptr : word;
oldnum : word;
begin
  GlobalEvent := false;
  { need to add protection from over allocation }
  oldnum := ReadParamWord(0,geventsloc);
  if (num < oldnum +2) then { we are re-writing an existing event or adding one }
  begin
    GlobalEvent := true;
    eventptr := gevent1loc + eventblocksize*(num-1);
    WriteParamWord(0,eventptr+operptr,nullLoc); {disable output part of event until done }
    with evrec do
    begin
      WriteParamWord(0,eventptr+condptr,ER_condptr);
      WriteParamWord(0,eventptr+condxor,ER_condxor);
      WriteParamWord(0,eventptr+condand,ER_condand_ptr);
      WriteParamWord(0,eventptr+condflag,ER_condflag);
      WriteParamWord(0,eventptr+operxor,ER_operxor);
      WriteParamWord(0,eventptr+operand,ER_operand);
      WriteParamWord(0,eventptr+operor,ER_operor_src);
      WriteParamWord(0,eventptr+operptr,ER_operptr);  {do last}
    end;
    if num > oldnum then WriteParamWord(0,geventsloc,num);
  end;
end;

{ These functions are for the optional trace buffer }

function ReadTraceBuffer : word;
begin
  ReadTraceBuffer := ReadParamWord(0,TbuffPopLoc); { get trace buffer data }
  WriteParamWord(0,TBuffPopLoc,0); { advance to next data }
end;

procedure WriteTraceBuffer(data : word);
begin
  WriteParamWord(0,TBuffPushLoc,data);
end;

function ReadTraceBuffCount : word;
begin
  ReadTraceBuffCount := ReadParamWord(0,TbuffCntLoc); { get trace data count }
end;

procedure ClearTraceBuffer;
begin
  WriteParamWord(0,TBuffCntLoc,0); { clear trace buffer }
end;

procedure EnableTBFIFOMode;
begin
  WriteParamWord(0,TBuffModeLoc,1); { set Stop On Full bit = FIFO mode }
end;

procedure EnableTBCircMode;
begin
  WriteParamWord(0,TBuffModeLoc,0); { clear Stop On FUll bit = Circular buffer mode }
end;


function GetTraceBufferSize: word;
var
index : word;
const
smallerthansmallest = 57;
biggerthanbiggest = 2049;
begin
  GetTraceBufferSize := 0;
  ClearTraceBuffer;
  EnableTBFIFOMode;
  for index := 1 to smallerthansmallest do { do some writes and check our count }
  begin
    WriteTraceBuffer(index);
  end;
  if ReadTraceBuffCount = smallerthansmallest then { count is ok so do a size test }
  begin
    ClearTraceBuffer;
    for index := 0 to biggerthanbiggest do
    begin
      WriteTraceBuffer(index);
    end;
    GetTraceBufferSize := ReadTraceBuffCount;
    for index := 0 to ReadTraceBuffCount -1 do
    begin
      if ReadTraceBuffer <> index then GetTraceBufferSize := 0;
    end;
  end;
end;

