library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity PhaseAcc is
   port ( 
      clk      : in  std_logic;
      ibus     : in  std_logic_vector (7 downto 0);
	  addr	   : in std_logic_vector (3 downto 0);
      loadfreq : in  std_logic;
      loadptr  : in  std_logic;
      loaddata  : in  std_logic;
      phaseout : out std_logic_vector (7 downto 0)

	);
end PhaseAcc;

architecture behavioral of PhaseAcc is
  component sinetable is
    port (
      addra : in  std_logic_vector(8 downto 0);
      addrb : in  std_logic_vector(8 downto 0);
      clka  : in  std_logic;
      clkb  : in  std_logic;
      dina  : in  std_logic_vector(7 downto 0);
      dinb  : in  std_logic_vector(7 downto 0);
      douta : out std_logic_vector(7 downto 0);
      doutb : out std_logic_vector(7 downto 0);
      wea   : in  std_logic;
      web   : in  std_logic);
  end component sinetable;


signal count: STD_LOGIC_VECTOR (47 downto 0);
signal latch: STD_LOGIC_VECTOR (47 downto 0);
signal ilatch0: STD_LOGIC_VECTOR (7 downto 0);
signal ilatch1: STD_LOGIC_VECTOR (7 downto 0);
signal ilatch2: STD_LOGIC_VECTOR (7 downto 0);
signal ilatch3: STD_LOGIC_VECTOR (7 downto 0);
signal ilatch4: STD_LOGIC_VECTOR (7 downto 0);
signal ilatch5: STD_LOGIC_VECTOR (7 downto 0);
signal ptrlatch: STD_LOGIC_VECTOR (8 downto 0);
signal doupdate : STD_LOGIC;

begin
	waveram: sinetable port map
	(
      addra => ptrlatch,
      addrb => count(47 downto 39),
      clka =>  clk,
      clkb =>  clk,
      dina => ibus,
	  dinb => "00000000",
      doutb => PhaseOut,
      wea  =>  loaddata,
      web => '0' 
	);	


	aphaseacc: process 	  (clk)

	begin
		if clk'event and clk = '1' then -- per clk stuff
			count <= count + latch;	
			if loadfreq = '1' then
				case addr is
					when b"0000" => ilatch0 <= ibus;
					when b"0001" => ilatch1 <= ibus;
					when b"0010" => ilatch2 <= ibus;
					when b"0011" => ilatch3 <= ibus;
					when b"0100" => ilatch4 <= ibus;
					when b"0101" => ilatch5 <= ibus;
									doupdate<= '1';
					when others => null;
				end case;
			end if; -- LoadFreq

			if loadPtr = '1' then
				case addr is
					when b"1000" => ptrlatch(7 downto 0) <= ibus;
					when b"1001" => ptrlatch(8) <= ibus(0);
					when others => null;
				end case;
			end if; -- loadptr

			if doupdate = '1' then -- update frequency latch from 8 bit latches
				latch(47 downto 40) <= ilatch5;
				latch(39 downto 32) <= ilatch4;
				latch(31 downto 24) <= ilatch3;
				latch(23 downto 16) <= ilatch2;
				latch(15 downto  8) <= ilatch1;
				latch( 7 downto  0) <= ilatch0;
				doupdate <= '0';		
			end if; -- doupdate
		end if; -- clk	
	end process;
end behavioral;
