{$N+}
program SC7I60; { added rev C compatability }

uses dos,crt;
const timeout = 10000000; { needs to be long enough for erase block }

type LongIntRec = record
  LowWord  : word;
  HighWord : word;
end;

type WordRec = record
  LowByte : byte;
  HighByte : byte;
end;

type DoubleIntRec = record
  Word0  : word;
  Word1  : word;
  Word2  : word;
  Word3  : word;
end;

type DoublelongRec = record
  long0  : longint;
  long1  : longint;
end;



type configfile = File;


const
BufferSize = 4096;
Longtime = 10000;
var

Revision : byte;
OurPageSize : word; { size to program, set conditionally }
OurSectorSize : longint; { size to erase, set conditionally }

BlockCount : longint;
PageCount : word;
Count : longint;
FileName : string;
CFile : configfile;
NeedsBitRev : boolean;
TheComPort : word;
TheBaudRate: longint;
BBuf: array[1..BufferSize] of byte;
RBuf: array[1..BufferSize] of byte;
BChecksum, RChecksum : byte;
DoReadback : boolean;
BusIntfc : boolean;

{$I ParseP.pas}
{$I serhost4.pas}

function SerialCheck : boolean;
begin
  SerialCheck := true;
  if SerError then
  begin
    SerialCheck := false;
    Writeln('Serial NAK Error');
    SerSync(TheComPort);
  end;
end;

procedure BarfOut(s: string);
begin
  writeln;
  writeln(s,chr(7));
  halt(2);
end;

function OpenForRead(var fp: configfile; name: string): boolean;
begin
  Assign(fp,Name);
  {$I-}
  Reset(fp);
  {$I+}
  OpenForRead := IOResult = 0;
end { Open };

procedure Usage;
begin
  writeln;
  writeln(' SC7I60 - Writes config data to 7I60');
  writeln;
  writeln(' USAGE - SC7I60 file port baud [-r] ');
  writeln(' Where file is a FPGA configuration file');
  writeln(' port is a COM port address and baud is a baud rate');
  writeln(' optional -r parameter does block readback for checking');
  writeln;
  halt(2);
end;

function Reverse(inbyte : byte) : byte;
const
revtable : array[0..$FF] of byte =

         ($00,$80,$40,$C0,$20,$A0,$60,$E0,$10,$90,$50,$D0,$30,$B0,$70,$F0,
          $08,$88,$48,$C8,$28,$A8,$68,$E8,$18,$98,$58,$D8,$38,$B8,$78,$F8,
          $04,$84,$44,$C4,$24,$A4,$64,$E4,$14,$94,$54,$D4,$34,$B4,$74,$F4,
          $0C,$8C,$4C,$CC,$2C,$AC,$6C,$EC,$1C,$9C,$5C,$DC,$3C,$BC,$7C,$FC,
          $02,$82,$42,$C2,$22,$A2,$62,$E2,$12,$92,$52,$D2,$32,$B2,$72,$F2,
          $0A,$8A,$4A,$CA,$2A,$AA,$6A,$EA,$1A,$9A,$5A,$DA,$3A,$BA,$7A,$FA,
          $06,$86,$46,$C6,$26,$A6,$66,$E6,$16,$96,$56,$D6,$36,$B6,$76,$F6,
          $0E,$8E,$4E,$CE,$2E,$AE,$6E,$EE,$1E,$9E,$5E,$DE,$3E,$BE,$7E,$FE,
          $01,$81,$41,$C1,$21,$A1,$61,$E1,$11,$91,$51,$D1,$31,$B1,$71,$F1,
          $09,$89,$49,$C9,$29,$A9,$69,$E9,$19,$99,$59,$D9,$39,$B9,$79,$F9,
          $05,$85,$45,$C5,$25,$A5,$65,$E5,$15,$95,$55,$D5,$35,$B5,$75,$F5,
          $0D,$8D,$4D,$CD,$2D,$AD,$6D,$ED,$1D,$9D,$5D,$DD,$3D,$BD,$7D,$FD,
          $03,$83,$43,$C3,$23,$A3,$63,$E3,$13,$93,$53,$D3,$33,$B3,$73,$F3,
          $0B,$8B,$4B,$CB,$2B,$AB,$6B,$EB,$1B,$9B,$5B,$DB,$3B,$BB,$7B,$FB,
          $07,$87,$47,$C7,$27,$A7,$67,$E7,$17,$97,$57,$D7,$37,$B7,$77,$F7,
          $0F,$8F,$4F,$CF,$2F,$AF,$6F,$EF,$1F,$9F,$5F,$DF,$3F,$BF,$7F,$FF);
begin
  Reverse := revtable[inbyte];
end;

function ReadString(var idx: word) : string;
var
sleng,count,index : word;
tstring : string;
begin
  index := idx;
  sleng := BBuf[index] * 256 + BBuf[index+1];
  index := index +2; {skip over string length}
  if sleng >255 then sleng := 255;
  tstring := '';
  for count := 1 to sleng do
  begin
    if BBuf[index] <> 0 then tstring := tstring + chr(BBuf[index]);
    index := index +1;
  end;
  idx := index;
  ReadString := tstring;
end;

function PrintBitFileHeader : word;
var
index,sleng : word;
tstring : string;
conflength : longint;
begin
  NeedsBitRev := false;
  index := 1;
  if (BBuf[index] = 0) and (BBuf[index+1] = 9) then
  begin
    writeln('Looks like a BIT file');
    index := index+2 +9; {skip over 00 09 and 9 header bytes}
    if (BBuf[index] = 0) and (BBuf[index+1] = 1) and (BBuf[index+2] = $61) then
    begin
      index := index + 3; { skip over 00 01 61 = 'a'}
      writeln('Design name: ',ReadString(index));
      index := index +1; {Skip over 'b'}
      writeln('Part name: ',ReadString(index));
      index := index +1; {Skip over 'c'}
      writeln('Design date: ',ReadString(index));
      index := index +1; {Skip over 'd'}
      writeln('Design time: ',ReadString(index));
      index := index +1; {Skip over 'e'}
      conflength := longint(BBuf[index])*16777216;
      conflength := conflength +longint(BBuf[index+1])*65536;
      conflength := conflength +longint(BBuf[index+2])*256;
      conflength := conflength +longint(BBuf[index+3]);
      writeln('Config Length: ',conflength);
      NeedsBitRev := true;
    end {valid bitfile header}
    else
    begin
      writeln('Invalid bitfile header!');
      halt(2);
    end;
  end {looks like a bit file }
  else
  begin
    if (BBuf[index] = $FF) and (BBuf[index+1] = $FF) and (BBuf[index+2] = $FF) and (BBuf[index+3] = $FF)  then
    begin
      if (BBuf[index+4] = $55) and (BBuf[index+5] = $99) and (BBuf[index+6] = $AA) and (BBuf[index+7] = $66)  then
      begin
        writeln('Looks like a PROM file');
      end
      else
      begin
        writeln('Invalid PROM file: No sync');
        halt(2);
      end;
    end
    else
    begin
      writeln('Invalid PROM file: no FF FF FF FF');
      halt(2);
    end;
  end;
  PrintBitFileHeader := index;
end;

procedure GetParms;
var
theport : word;
thefile : string;
tempstr : string;
templong : longint;
retcode : integer;
  begin
  if ParamCount < 3 then Usage;
  begin
    thefile := paramStr(1);
  end;
  FileName := thefile;
  if not OpenForRead(CFile,FileName) then
  begin
    writeln('Can''t open configuration file');
    halt(2);
  end;
  tempstr := ParamStr(2);
  tempstr := UpString(tempstr);
  if not HexLongRead(tempstr,templong) then
  begin
    TheComPort := 0;
    if tempstr = 'COM1' then TheComPort := COM1;
    if tempstr = 'COM2' then TheComPort := COM2;
    if tempstr = 'COM3' then TheComPort := COM3;
    if tempstr = 'COM4' then TheComPort := COM4;
    if TheComPort = 0 then BarfOut('Undecipherable COM port');
  end
  else
  begin
    if (templong <$100) or (templong >$3F8) then
    begin
      BarfOut('Out of range COM port address!');
    end;
    if (templong and 7) <> 0 then
    begin
      BarfOut('Illegal COM port address!');
    end;
    TheComPort := word(templong);
  end;
  if length(ParamStr(3)) <> 0 then
  begin
    tempstr := ParamStr(3);
    val(tempstr,templong,retcode);
    if (templong <300) or (templong >115200) or (retcode <> 0) then
    begin
      BarfOut('Bad baud rate parameter!');
    end
    else
    begin
      TheBaudRate := templong;
    end;
  end;
  if length(ParamStr(4)) <> 0 then DoReadBack := true else DoReadback := false;
  InitializeSerInterface(TheComPort,TheBaudRate);
  delay(100);
  TossChars(TheComPort,20);
end;

Procedure ShowErrors;
var
index : word;
foostr : string;
begin
  for index := 1 to OurPageSize do
  begin
    if BBuf[index] <> Rbuf[index] then
    begin
      write('Data Error at byte ');
      HexPrint(index,2);
      write(': Expected ');
      HexPrint(BBuf[index],2);
      write(' but got back ');
      Hexprint(RBuf[index],2);
      writeln;
      writeln('Press return to continue');
      readln(foostr);
    end;
  end;
end;

function CalcCheckSum(block: word) : byte;
var
index: longint;
sum : byte;
hexstr : string;
begin
 sum := 0;
 if revision < 64 then
 begin
   hexstr := Hexstring(block,2);
   sum := byte(hexstr[1]);
   sum := sum + byte(hexstr[2]);
 end
 else
 begin
   hexstr := Hexstring(block,4);
   sum := byte(hexstr[1]);
   sum := sum + byte(hexstr[2]);
   sum := sum + byte(hexstr[3]);
   sum := sum + byte(hexstr[4]);
  end;
  for index := 1 to OurPageSize do
  begin
    hexstr := Hexstring(BBuf[index],2);
    sum := sum + byte(hexstr[1]);
    sum := sum + byte(hexstr[2]);
  end;
  CalcCheckSum := sum;
end;

procedure DoIt;
var
bytesread,index : word;
bytecount : longint;
firstblock : boolean;
blocknum : word;
retchar : char;
hstring : string;
begin
  writeln;
  if OpenForRead(CFile,FileName) then
  begin
    blocknum := 0;
    Count := (filesize(CFile)*128) + 128;
    SerUnlock(TheComport);
    while Count >0 do
    begin
        writeln('Erasing ',OurSectorSize,' bytes in block# ',blocknum);
        EraseFlashBlock(TheComPort,blocknum);
        Count := Count - OursectorSize;
        blocknum := blocknum + 1;
    end;

    { now program needed blocks }
    blocknum := 0;
    Count := 0;
    FirstBlock := true;
    SerUnlock(TheComport);
    reset(cfile,1);
    while not EOF(CFile) do
    begin
      bytesread := 0;
      blockread(CFile,BBuf,OurPageSize,bytesread);
      if firstblock then PrintBitFileHeader;
      firstblock := false;
      if bytesread > 0 then
      begin
        Write('WB# ',blocknum,'  ',chr(13));
        ClearChecksum(TheComPort);
        BChecksum := CalcChecksum(blocknum);
        if revision < 64 then WriteFlashBlock(TheComPort,blocknum,@BBuf)
        else WriteNewFlashBlock(TheComPort,blocknum,OurPageSize,@BBuf);
        RChecksum := ReadChecksum(TheComPort);
        if Rchecksum <> BChecksum then
        begin
          writeln('Checksum error');
          writeln('BCS= ',BChecksum,' RCS= ',RChecksum);
        end;
        if DoReadback then
        begin
          Write('RB# ',blocknum,'  ',chr(13));
          if revision < 64 then ReadFlashBlock(TheComPort,blocknum,@RBuf)
          else ReadNewFlashBlock(TheComPort,blocknum,OurPageSize,@RBuf);
          ShowErrors;
        end;
      end; { bytes read }
    blocknum := blocknum +1;
    end;
    SerLock(TheComPort);
  end
  else
  begin
    writeln('Can''t open configuration file');
    halt(2);
  end;
  writeln;
end;

begin
  GetParms;
  if not SerSync(TheComport) then Barfout('7I60 communication error');
  writeln('Card type is ',InquireID(TheComport));
  writeln('Pic Revision is ',InquireRev(TheComport));
  Revision := InquireRev(TheComport);

  if Revision < 64 then
    begin { old 45f eeprom }
      OurPageSize := 4096;
      OurSectorSize := 4096;
    end
  else { new 25p eeprom }
    begin
      OurPageSize := 256;
      OurSectorSize := 65536;
    end;
    DoIt;
  writeln('Configuration complete!');
  writeln('Doing FPGA config');
  ConfigureFPGA(TheComport);
  halt(0);
end.
