{for fifo'ed 7I60}
{$R-}    {Range checking be off }
{$I-}    {No I/O checking }
{$S-}    {No stack checking}

{$I Serial.pas}      { include low level serial routines }
{$I hread.pas}
{$I hprint.pas}
const
  CR = chr(13);
  NAK = '?';
  FlashBlockSize = 4096;
  SerReadChecksumCom =     'RC';
  SerReadEEPROMCom =       'RE';
  SerReadFlashCom =        'RF';
  SerReadProtectCom =      'RP';
  SerReadROMCom =          'RR';
  SerReadGroupCom =        'RG';

  SerReadIFIFOCom =        'RI';
  SerReadQFIFOCom =        'RQ';
  SerReadIFIFOMultipleCom ='RM';
  SerReadQFIFOMultipleCom ='Rm';
  SerReadFIFOStatusCom =   'Rs';
  SerReadDirectWordCom =   'Rd'; { for fifoed interface }
  SerReadDirectByteCom =   'RD';

  SerWriteEEPROMCom =      'WE';
  SerWriteFlashCom =       'WF';
  SerWriteProtectCom =     'WP';
  SerWriteOscCom     =     'WO';
  SerWriteROMCom =         'WR';
  SerWriteDirectWordCom =  'Wd';  { for fifoed interface }
  SerWriteDirectByteCom =  'WD';
  SerWriteGroupCom =       'WG';

  SerWriteIFIFOCom =       'WI';
  SerWriteQFIFOCom =       'WQ';

  SerEraseFlashCom =       'EF';
  SerEraseChecksumCom =    'EC';
  SerEraseGroupCom =       'EG';

  SerInquireIDCom  =    'II';
  SerInquireRevCom =    'IR';

  SerPICGoCom      =    'GP';
  SerConfigCom     =    'GC';

  SerClearICDFIFOCom =  'CI';   { not 7I60 yet }
  SerClearIRBFIFOCom =  'Ci';
  SerClearQCDFIFOCom =  'CQ';
  SerClearQRBFIFOCom =  'Cq';

var
SerError : boolean;
GroupSize  : word;

type buffer = array[1.. FlashBlockSize] of byte; { largest size }
     bufptr = ^buffer;
     DataBuffertype = array[0..255] of word;
     DataBuffPtr = ^DataBuffertype;

var  DataBuffer : DataBuffertype;

{*****************************************************************************}
function GetComEnv: boolean;
var
retcode : integer;
portstr,baudstr,busstr : string;
port : word;
baud : longint;
comport : longint;
begin
  GetComEnv := true;
  portstr := UpString(GetEnv('COMPORT'));
  baudstr := GetEnv('BAUDRATE');
  busstr := GetEnv('BUSINTFC');
  if length(portstr) = 0 then Writeln('Comport not in Environment');
  if length(baudstr) = 0 then Writeln('Baudrate not in Environment');
  if length(busstr) = 0 then Writeln('Busintfc not in Environment');
  if (length(portstr) = 0) or (length(baudstr) = 0) or (length(busstr) = 0) then
    GetComEnv := false;
  if length(busstr) <> 0 then
  begin
    if upcase(busstr[1]) = 'Y' then BusIntfc := true;
    if upcase(busstr[1]) = 'T' then BusIntfc := true;
  end;
  if BusIntfc = false then
  begin
    if not HexLongRead(portstr,comport) then
    begin
      comport := 0;
      if portstr = 'COM1' then comport := COM1;
      if portstr = 'COM2' then comport := COM2;
      if portstr = 'COM3' then comport := COM3;
      if portstr = 'COM4' then comport := COM4;
      if comport = 0 then
      begin
        Writeln('Undecipherable COM port - bail out!');
        halt(2);
      end;
    end
    else
    begin
      if (comport <$100) or (comport >$3F8) then
      begin
        writeln('Out of range COM port address - bail out!');
        halt(2);
      end;
      if (comport and 7) <> 0 then
      begin
        Writeln('Illegal COM port address - bail out!');
        halt(2);
      end;
    end;
    TheComPort := word(comport);
    if length(baudstr) <> 0 then
    begin
      val(baudstr,baud,retcode);
      if (baud <300) or (baud >115200) or (retcode <> 0) then
      begin
        Writeln('Illegal Baud rate - bail out!');
        halt(2);
      end;
    end;
    TheBaudRate := baud;
  end;
end;

procedure SendString(tcp : word; s: string);
var index : word;
begin
  for index := 1 to length(s) do
  begin
    PolledPutComChar(tcp,s[index]);
  end;
end;

procedure SendStringWithCTSHandshakeAndTimeout(tcp : word; s: string);
var index : word;
begin
  for index := 1 to length(s) do
  begin
    PolledPutComCharWithCTSHandshakeAndTimeout(tcp,s[index],timeout);
  end;
end;

function RecvString(tcp : word; sz: word) : string;
var
index : word;
tstring : string;
achar : char;
begin
  tstring := '';
  index := 1;
  while (index <= sz) and (not SerError) do
  begin
    if PolledGetComCharWithTimeout(tcp,achar,timeout) = 0 then SerError := true;
    if achar = NAK then SerError := true;
    tstring := tstring + achar;
    index := index + 1
  end;
  RecvString := tstring;
end;

procedure InitializeSerInterface(comport: word;baudrate:real);
begin
  DefaultComInit(comport);
  EnableFifos(comport); { don't check if fifos are there because it sends garbage }
  SetBaudRate(comport,baudrate);
  TossChars(comport,130);
end;

procedure SerWriteDirectByte(comport: word; address:byte; data:byte);
var
sstring : string;
retchar : char;
begin
  sstring := SerWritedirectByteCom+HexString(address,2)+HexString(data,2)+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure SerWriteDirectWord(comport: word; address:byte; data:word);
var
sstring : string;
retchar : char;
begin
  sstring := SerWritedirectWordCom+HexString(address,2)+HexString(data,4)+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;


procedure WriteGroup(comport : word; group : byte;ourbuffer : bufptr);
var
sstring : string;
retchar : char;
index : word;
begin
  sstring := SerWriteGroupCom+ HexString(group,4)+CR;
  SendString(comport,sstring);
  for index := 1 to GroupSize do SendString(comport,HexString(ourbuffer^[index],2));
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

function SerReadIFIFO(comport: word) : word;
var
data : word;
sstring : string;
retchar : char;
begin
  sstring := SerReadIFIFOCom + CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,4);  { get 4 char hex string }
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true; { get CR }
  HexWordRead(sstring,data);
  SerReadIFIFO := data;
end;

procedure SerReadQFIFOMultiple(comport: word; n : word; bufptr : DataBuffPtr);
var
data : word;
index : byte;
sstring : string;
retchar : char;
begin
  sstring := SerReadQFIFOMultipleCom + HexString(n,2) + CR;
  SendString(comport,sstring);
  for index := 0 to n-1 do
  begin
    HexWordRead(RecvString(comport,4),data);
    bufptr^[index] := data;
  end;
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true; { get CR }
end;

function SerReadQFIFO(comport: word) : word;
var
data : word;
sstring : string;
retchar : char;
begin
  sstring := SerReadQFIFOCom + CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,4);  { get 4 char hex string }
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true; { get CR }
  HexWordRead(sstring,data);
  SerReadQFIFO := data;
end;

function SerReadDirectByte(comport: word; address: byte) : byte;
var
data : word;
sstring : string;
retchar : char;
begin
  sstring := SerReadDirectByteCom + HexString(address,2) +CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,2);  { get 4 char hex string }
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true; { get CR }
  HexWordRead(sstring,data);
  SerReadDirectByte := data;
end;

function SerReadDirectWord(comport: word; address: byte) : word;
var
data : word;
sstring : string;
retchar : char;
begin
  sstring := SerReadDirectWordCom + HexString(address,2) +CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,4);  { get 4 char hex string }
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true; { get CR }
  HexWordRead(sstring,data);
  SerReadDirectWord := data;
end;

function SerReadStatus(comport: word) : word;
var
data : word;
sstring : string;
retchar : char;
begin
  sstring := SerReadFifoStatusCom + CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,4);  { get 4 char hex string }
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true; { get CR }
  HexWordRead(sstring,data);
  SerReadStatus := data;
end;

function SerReadParamWord(comport: word;command:word) : word;
var
data : word;
sstring : string;
retchar : char;
begin
  sstring := SerWriteIFIFOCom + HexString(command,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true; { get CR }
  sstring := SerReadIFIFOCom + CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,4);  { get 4 char hex string }
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true; { get CR }
  HexWordRead(sstring,data);
  SerReadParamWord := data;
end;

function SerReadParam(comport: word;command:word) : longint;
var
sstring : string;
rlstring : string;
rhstring : string;
ldata : longint;
retchar : char;
begin
  sstring := SerWriteIFIFOCom + HexString(command,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerReadIFIFOCom + CR;
  SendString(comport,sstring);
  rlstring := RecvString(comport,4);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  SendString(comport,sstring);
  rhstring := RecvString(comport,4);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  HexLongRead(rhstring+rlstring,ldata);
  SerReadParam := ldata;
end;

function SerReadParamDouble(comport: word;command:word) : comp;
var
sstring : string;
r0string,r1string,r2string,r3string : string;
cdata : comp;
retchar : char;
begin
  sstring := SerWriteIFIFOCom + HexString(command,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerReadIFIFOCom + CR;
  SendString(comport,sstring);
  r0string := RecvString(comport,4);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  SendString(comport,sstring);
  r1string := RecvString(comport,4);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  SendString(comport,sstring);
  r2string := RecvString(comport,4);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  SendString(comport,sstring);
  r3string := RecvString(comport,4);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  HexLongRead(r0string+r1string,DoubleLongRec(cdata).Long0);
  HexLongRead(r2string+r3string,DoubleLongRec(cdata).Long1);
  SerReadParamDouble := cdata;
end;

procedure SerWriteParamWord(comport: word;command:word;data:word);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteIFIFOCom+HexString(command,4)+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerWriteIFIFOCom+HexString(data,4)+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure SerWriteParam(comport:word;command: word;ldata:longint);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteIFIFOCom + HexString(command,4)+ CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerWriteIFIFOCom + HexString(LongIntRec(ldata).LowWord,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerWriteIFIFOCom + HexString(LongIntRec(ldata).HighWord,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure SerWriteParamDouble(comport:word;command: word;cdata:comp);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteIFIFOCom + HexString(command,4)+ CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerWriteIFIFOCom + HexString(DoubleIntRec(cdata).Word0,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerWriteIFIFOCom + HexString(DoubleIntRec(cdata).Word1,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerWriteIFIFOCom + HexString(DoubleIntRec(cdata).Word2,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerWriteIFIFOCom + HexString(DoubleIntRec(cdata).Word3,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

function SerReadParamWordQ(comport: word;command:word) : word;
var
data : word;
sstring : string;
retchar : char;
begin
  sstring := SerWriteQFIFOCom + HexString(command,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true; { get CR }
  sstring := SerReadQFIFOCom + CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,4);  { get 4 char hex string }
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true; { get CR }
  HexWordRead(sstring,data);
  SerReadParamWordQ := data;
end;

function SerReadParamQ(comport: word;command:word) : longint;
var
sstring : string;
rlstring : string;
rhstring : string;
ldata : longint;
retchar : char;
begin
  sstring := SerWriteQFIFOCom + HexString(command,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerReadQFIFOCom + CR;
  SendString(comport,sstring);
  rlstring := RecvString(comport,4);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  SendString(comport,sstring);
  rhstring := RecvString(comport,4);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  HexLongRead(rhstring+rlstring,ldata);
  SerReadParamQ := ldata;
end;

function SerReadParamDoubleQ(comport: word;command:word) : comp;
var
sstring : string;
r0string,r1string,r2string,r3string : string;
cdata : comp;
retchar : char;
begin
  sstring := SerWriteQFIFOCom + HexString(command,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerReadQFIFOCom + CR;
  SendString(comport,sstring);
  r0string := RecvString(comport,4);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  SendString(comport,sstring);
  r1string := RecvString(comport,4);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  SendString(comport,sstring);
  r2string := RecvString(comport,4);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  SendString(comport,sstring);
  r3string := RecvString(comport,4);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  HexLongRead(r0string+r1string,DoubleLongRec(cdata).Long0);
  HexLongRead(r2string+r3string,DoubleLongRec(cdata).Long1);
  SerReadParamDoubleQ := cdata;
end;

procedure SerWriteParamWordQ(comport: word;command:word;data:word);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteQFIFOCom+HexString(command,4)+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerWriteQFIFOCom+HexString(data,4)+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure SerWriteParamQ(comport:word;command: word;ldata:longint);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteQFIFOCom + HexString(command,4)+ CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerWriteQFIFOCom + HexString(LongIntRec(ldata).LowWord,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerWriteIFIFOCom + HexString(LongIntRec(ldata).HighWord,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure SerWriteParamDoubleQ(comport:word;command: word;cdata:comp);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteQFIFOCom + HexString(command,4)+ CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerWriteQFIFOCom + HexString(DoubleIntRec(cdata).Word0,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerWriteQFIFOCom + HexString(DoubleIntRec(cdata).Word1,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerWriteQFIFOCom + HexString(DoubleIntRec(cdata).Word2,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  sstring := SerWriteQFIFOCom + HexString(DoubleIntRec(cdata).Word3,4) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure ReadGroup(comport : word; group : byte;ourbuffer : bufptr);
var
sstring : string;
retchar : char;
index : word;
begin
  sstring := SerReadGroupCom+ HexString(group,4)+CR;
  SendString(comport,sstring);
  for index := 1 to GroupSize do
  begin
    HexByteRead(RecvString(comport,2),ourbuffer^[index]);
  end;
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure ClearChecksum(comport : word);
var
sstring : string;
retchar : char;
begin
  sstring := SerEraseChecksumCom+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

function ReadChecksum(comport: word) : byte;
var
sstring : string;
data : byte;
retchar : char;
begin
  sstring := SerReadChecksumCom + CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,2);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  HexbyteRead(sstring,data);
  ReadChecksum := data;
end;

procedure SerUnlock(comport : word);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteProtectCom+'00'+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure SerLock(comport : word);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteProtectCom+ 'FF'+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure SerPicGo(comport: word; address:word);
var
sstring : string;
retchar : char;
begin
  SerUnlock(comport);
  sstring := SerPicGoCom+HexString(address,4)+CR;
  SendString(comport,sstring);
 { if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;}
end;

procedure SerClearICDFIFO(comport : word);
var
sstring : string;
retchar : char;
begin
  sstring := SerClearICDFIFOCom+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure SerClearQCDFIFO(comport : word);
var
sstring : string;
retchar : char;
begin
  sstring := SerClearQCDFIFOCom+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure SerClearIRBFIFO(comport : word);
var
sstring : string;
retchar : char;
begin
  sstring := SerClearIRBFIFOCom+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure SerClearQRBFIFO(comport : word);
var
sstring : string;
retchar : char;
begin
  sstring := SerClearQRBFIFOCom+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure WriteFlashBlock(comport : word; block : byte;ourbuffer : bufptr);
var
sstring : string;
retchar : char;
index : word;
begin
  sstring := SerWriteFlashCom + HexString(block,2)+CR;
  SendString(comport,sstring);
  for index := 1 to FlashBlockSize do
  begin
    SendString(comport,HexString(ourbuffer^[index],2))
  end;
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure WriteNewFlashBlock(comport : word; block : word; size : word; ourbuffer : bufptr);
var
sstring : string;
retchar : char;
index : word;
begin
  sstring := SerWriteFlashCom+ HexString(block,4)+CR;
  SendString(comport,sstring);
  for index := 1 to size do
  begin
    SendString(comport,HexString(ourbuffer^[index],2));
  end;
  if PolledGetComCharWithTimeout(comport,retchar,timeout) = 0 then SerError := true;
end;

procedure EraseFlashBlock(comport : word; block : byte);
var
sstring : string;
retchar : char;
index : word;
begin
  sstring := SerEraseFlashCom+ HexString(block,2)+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure ReadFlashBlock(comport : word; block : byte;ourbuffer : bufptr);
var
sstring : string;
retchar : char;
index : word;
begin
  sstring := SerReadFlashCom+ HexString(block,2)+CR;
  SendString(comport,sstring);
  for index := 1 to FlashBlockSize do
  begin
    HexByteRead(RecvString(comport,2),ourbuffer^[index]);
  end;
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure ReadNewFlashBlock(comport : word; block : byte; size : word; ourbuffer : bufptr);
var
sstring : string;
retchar : char;
index : word;
begin
  sstring := SerReadFlashCom+ HexString(block,4)+CR;
  SendString(comport,sstring);
  for index := 1 to size do
  begin
    HexByteRead(RecvString(comport,2),ourbuffer^[index]);
  end;
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

function InquireID(comport: word) :string;
var
sstring : string;
data : byte;
retchar : char;
begin
  sstring := SerInquireIDCom +CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,4);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  InquireID := sstring;
end;

function InquireRev(comport: word) :byte;
var
sstring : string;
data : byte;
retchar : char;
begin
  sstring := SerInquireRevCom +CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,2);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  HexbyteRead(sstring,data);
  InquireRev := data;
end;

procedure WriteEEPROM(comport:word;addr: byte;data:byte);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteEEPROMCom + HexString(addr,2)+ HexString(data,2) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

function ReadEEPROM(comport: word;add:byte) : byte;
var
sstring : string;
data : byte;
retchar : char;
begin
  sstring := SerReadEEPROMCom +HexString(add,2)+ CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,2);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
  HexbyteRead(sstring,data);
  ReadEEPROM := data;
end;

procedure EraseGroup(comport : word; block : byte);
var
sstring : string;
retchar : char;
index : word;
begin
  sstring := SerEraseGroupCom+ HexString(block,4)+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure WritePLL(comport:word;data:byte);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteOscCom + HexString(data,2) + CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

procedure ConfigureFPGA(comport : word);
var
sstring : string;
retchar : char;
begin
  sstring := SerConfigCom+CR;
  SendString(comport,sstring);
  if PolledGetComCharWithTimeout(comport,retchar,timeout) =0 then SerError := true;
end;

function SerSync(comport : word) : boolean;
var
retchar : char;
sstring : string;
begin
  SerSync := false;
  ForceRTSHigh(comport);
  PolledPutComChar(ComPort,'x'); { send invalid command or data }
  PolledPutComChar(ComPort,CR);
  if PolledGetComCharWithTimeout(ComPort,retchar,timeout) <> 0 then
  begin
    TossChars(comport,130);
    PolledPutComChar(ComPort,'x'); { send invalid command or data }
    PolledPutComChar(ComPort,CR);
    if PolledGetComCharWithTimeout(ComPort,retchar,timeout) <> 0 then
    begin
      TossChars(comport,130);
      if retchar = '?' then
      begin
        sstring := SerWriteProtectCom +'AA' + CR;
        SendString(ComPort,sstring);
        if PolledGetComCharWithTimeout(ComPort,retchar,timeout) <> 0 then
        begin
          if retchar = CR then
          begin
            SerSync := true;
            SerError := false;
          end
        end;
      end;
    end;
  end;
end;
