program Looptest;
uses crt;
{$I serhost}
{$I parsep}

const
PortBase = $C0;
NPorts = 12;
DDROffset = 1;
var
TheComport : word;
TheBaudrate : longint;

procedure BarfOut(s: string);
begin
  writeln;
  writeln(s,chr(7));
  halt(2);
end;

procedure Usage;
begin
  writeln;
  writeln(' 7I60LOOP - TESTS 7I60 I/O BITS');
  writeln;
  writeln(' USAGE - 7I60LOOP port baud ');
  writeln(' port is a COM port address and baud is a baud rate');
  writeln(' Assumes loopback cables from P4->P5 and P7->P8');
  writeln;
  halt(2);
end;

function DataFromPortNum(port: byte): byte;
begin
  DataFromPortNum := PortBase+((port-1) *2);
end;

function DDRFromPortNum(port: byte): byte;
begin
  DDRFromPortNum := PortBase+((port-1) *2)+DDROffset;
end;

function IOBitFromPortBit(port: byte; bit : byte): byte;
begin
  IOBitFromPortBit := ((port-1) * 8 + bit);
end;

function ConnFromPort(port: byte): String;
var tstring : string;
begin
  Case port of
    1,2,3 : tstring := 'P7';
    4,5,6 : tstring := 'P8';
    7,8,9 : tstring := 'P5';
    10,11,12 : tstring := 'P4';
  end {case};
  ConnFromPort :=  tstring;
end;


procedure GetParms;
var
theport : word;
tempstr : string;
templong : longint;
retcode : integer;
  begin
  if ParamCount < 2 then Usage;
  tempstr := ParamStr(1);
  tempstr := UpString(tempstr);
  if not HexLongRead(tempstr,templong) then
  begin
    TheComPort := 0;
    if tempstr = 'COM1' then TheComPort := COM1;
    if tempstr = 'COM2' then TheComPort := COM2;
    if tempstr = 'COM3' then TheComPort := COM3;
    if tempstr = 'COM4' then TheComPort := COM4;
    if TheComPort = 0 then BarfOut('Undecipherable COM port');
  end
  else
  begin
    if (templong <$100) or (templong >$3F8) then
    begin
      BarfOut('Out of range COM port address!');
    end;
    if (templong and 7) <> 0 then
    begin
      BarfOut('Illegal COM port address!');
    end;
    TheComPort := word(templong);
  end;
  if length(ParamStr(2)) <> 0 then
  begin
    tempstr := ParamStr(2);
    val(tempstr,templong,retcode);
    if (templong <300) or (templong >115200) or (retcode <> 0) then
    begin
      BarfOut('Bad baud rate parameter!');
    end
    else
    begin
      TheBaudRate := templong;
    end;
  end;
  InitializeSerInterface(TheComPort,TheBaudRate);
  delay(100);
  TossChars(TheComPort,20);
end;

procedure SetAllOut;
var
port : byte;
begin
  for port := 1 to NPorts do
  begin
    WriteDirect(TheComPort,DDRFromPortNum(port),$FF);
  end;
end;

procedure SetAllIn;
var
port : byte;
begin
  for port := 1 to NPorts do
  begin
    WriteDirect(TheComPort,DDRFromPortNum(port),$00);
  end;
end;

procedure WritePortNumbers;
var
port : byte;
begin
  SetallIn;
  for port := 1 to NPorts do
  begin
    WriteDirect(TheComPort,DataFromPortNum(port),port);
  end;
end;

procedure CheckPortNumbers;
var
port : byte;
begin
  SetallIn;
  for port := 1 to NPorts do
  begin
    WriteDirect(TheComPort,DDRFromPortNum(port),$FF); { set port to output mode }
    if ReadDirect(TheComPort,DataFromPortNum(port)) <> port then Barfout('Port addressing problem');
    WriteDirect(TheComPort,DDRFromPortNum(port),$00); { set port back to input mode }
  end;
end;

procedure CheckResetState;
const
bitmask = $01;
eportdata = $FF;
eddrdata = $00;
var
errors,port : byte;
tmask,bit,portdata,ddrdata : byte;

begin
  for port := 1 to NPorts do
  begin
    portdata := ReadDirect(TheComPort,DataFromPortNum(port));
    ddrdata := ReadDirect(TheComPort,DDRFromPortNum(port));
    if portdata <> eportdata then
    begin
      errors := portdata xor eportdata;
      for bit := 0 to 7 do
      begin
        tmask := bitmask shl bit;
        if (errors and tmask) <> 0 then
        begin
          writeln('Initial bit error on ',ConnFromPort(port),' IO',IOBitFromPortBit(port,bit));
          barfout('7I60 bad');
        end;
      end;
    end;
    if ddrdata <> eddrdata then
    begin
      errors := portdata xor eportdata;
      for bit := 0 to 7 do
      begin
      tmask := bitmask shl bit;
        if (errors and tmask) <> 0 then
        begin
          writeln('Initial DDR error on port ',port,' bit ',bit);
          BarfOut('7I60 bad');
        end;
      end;
    end;
  end;
end;




procedure DoLoopback(porta, portb : byte);
const
bitmask = $01;
var
bit : byte;
begin
  SetAllIn;
  WriteDirect(TheComPort,DDRFromPortNum(porta),$FF);
  WriteDirect(TheComPort,DDRFromPortNum(portb),$00);
  for bit := 0 to 7 do
  begin
    WriteDirect(TheComport,DataFromPortNum(porta),(bitmask shl bit));
    if ReadDirect(TheComport,DataFromPortNum(portb)) <> (bitmask shl bit) then
    begin
      write('Error: send bit = ',ConnFromPort(porta),' IO',IOBitFromPortBit(porta,bit));
      writeln(' Recv bit = ',ConnFromPort(portb),' IO',IOBitFromPortBit(portb,bit));
      BarfOut('7I60Bad');
    end;
  end;
end;

procedure DoLooptest;
begin
  DoLoopback(1,4);
  DoLoopback(4,1);
  DoLoopback(2,5);
  DoLoopback(5,2);
  DoLoopback(3,6);
  DoLoopback(6,3);
  DoLoopback(7,10);
  DoLoopback(10,7);
  DoLoopback(8,11);
  DoLoopback(11,8);
  DoLoopback(9,12);
  DoLoopback(12,9);
end;



begin
  GetParms;
  InitializeSerInterface(TheComport,TheBaudRate);
  if not SerSync(TheComport) then Barfout('7I60 communication error');
  CheckResetState;
  WritePortNumbers;
  CheckPortNumbers;
  DoLooptest;
  SetAllIn;
end.