{$R-}    {Range checking be off }
{$I-}    {No I/O checking }
{$S-}    {No stack checking}

{$I Serial.pas}      { include low level serial routines }
{$I hread.pas}
{$I hprint.pas}

const
  FlashBlockSize = 4096;
  CR = chr(13);
  SerReadBaseCom =      'RB';
  SerReadChecksumCom =  'RC';
  SerReadDirectCom =    'RD';
  SerReadEEPROMCom =    'RE';
  SerReadFlashCom =     'RF';
  SerReadLongCom =      'RL';
  SerReadProtectCom =   'RP';
  SerReadROMCom =       'RR';
  SerReadStatusCom =    'RS';
  SerReadWordCom =      'RW';

  SerWriteBaseCom =     'WB';
  SerWriteDirectCom =   'WD';
  SerWriteEEPROMCom =   'WE';
  SerWriteFlashCom =    'WF';
  SerWriteLongCom =     'WL';
  SerWriteProtectCom =  'WP';
  SerWriteOscillator =  'WO';
  SerWriteROMCom =      'WR';
  SerWriteStatusCom =   'WS';
  SerWriteWordCom =     'WW';

  SerEraseFlashCom =    'EF';
  SerEraseChecksumCom = 'EC';

  SerInquireIDCom  =    'II';
  SerInquireRevCom =    'IR';

  SerPICGoCom      =    'GO';
  SerConfigCom     =    'GC';

type buffer = array[1.. FlashBlockSize] of byte;
type bufptr = ^buffer;





{*****************************************************************************}



procedure SendString(tcp : word; s: string);
var index : word;
begin
  for index := 1 to length(s) do
  begin
    PolledPutComChar(tcp,s[index]);
  end;
end;

function RecvString(tcp : word; sz: word) : string;
var
index : word;
tstring : string;
achar : char;
begin
  tstring := '';
  for index := 1 to sz do
  begin
    PolledGetComChar(tcp,achar);
    tstring := tstring + achar;
  end;
  RecvString := tstring;
end;

procedure InitializeSerInterface(comport: word;baudrate:real);
begin
  DefaultComInit(comport);
  EnableFifos(comport); { don't check if fifos are there because it sends garbage }
  SetBaudRate(comport,baudrate);
  TossChars(comport,20);

end;

procedure SerWriteParamWord(comport: word;command:word;data:word);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteWordCom+HexString(command,4)+HexString(data,4)+CR;
  SendString(comport,sstring);
  PolledGetComChar(comport,retchar);
end;

function SerReadParamWord(comport: word;command:word) : word;
var
data : word;
sstring : string;
retchar : char;
begin
  sstring := SerReadWordCom + HexString(command,4) + CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,4);  { get 4 char hex string }
  PolledGetComChar(comport,retchar); { get CR }
  HexWordRead(sstring,data);
  SerReadParamWord := data;
end;

procedure SerWriteParam(comport:word;command: word;ldata:longint);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteLongCom + HexString(command,4)+ HexString(ldata,8) + CR;
  SendString(comport,sstring);
  PolledGetComChar(comport,retchar);
end;

function SerReadParam(comport: word;command:word) : longint;
var
sstring : string;
ldata : longint;
retchar : char;
begin
  sstring := SerReadLongCom + HexString(command,4) + CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,8);
  PolledGetComChar(comport,retchar);
  HexLongRead(sstring,ldata);
  SerReadParam := ldata;
end;

procedure ClearChecksum(comport : word);
var
sstring : string;
retchar : char;
begin
  sstring := SerEraseChecksumCom+CR;
  SendString(comport,sstring);
  PolledGetComChar(comport,retchar);
end;

function ReadChecksum(comport: word) : byte;
var
sstring : string;
data : byte;
retchar : char;
begin
  sstring := SerReadChecksumCom + CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,2);
  PolledGetComChar(comport,retchar);
  HexbyteRead(sstring,data);
  ReadChecksum := data;
end;

function ReadDirect(comport: word;add:byte) : byte;
var
sstring : string;
data : byte;
retchar : char;
begin
  sstring := SerReadDirectCom +HexString(add,2)+ CR;
  SendString(comport,sstring);
  sstring := RecvString(comport,2);
  PolledGetComChar(comport,retchar);
  HexbyteRead(sstring,data);
  ReadDirect := data;
end;

procedure WriteDirect(comport:word;addr: byte;data:byte);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteDirectCom + HexString(addr,2)+ HexString(data,2) + CR;
  SendString(comport,sstring);
  PolledGetComChar(comport,retchar);
end;

procedure SerUnlock(comport : word);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteProtectCom+'00'+CR;
  SendString(comport,sstring);
  PolledGetComChar(comport,retchar);
end;

procedure SerLock(comport : word);
var
sstring : string;
retchar : char;
begin
  sstring := SerWriteProtectCom+ 'FF'+CR;
  SendString(comport,sstring);
  PolledGetComChar(comport,retchar);
end;

procedure WriteFlashBlock(comport : word; block : byte;ourbuffer : bufptr);
var
sstring : string;
retchar : char;
index : word;
begin
  sstring := SerWriteFlashCom+ HexString(block,2)+CR;
  SendString(comport,sstring);
  for index := 1 to FlashBlockSize do
  begin
    SendString(comport,HexString(ourbuffer^[index],2))
  end;
  PolledGetComChar(comport,retchar);
end;

procedure EraseFlashBlock(comport : word; block : byte);
var
sstring : string;
retchar : char;
index : word;
begin
  sstring := SerEraseFlashCom+ HexString(block,2)+CR;
  SendString(comport,sstring);
  PolledGetComChar(comport,retchar);
end;

procedure ReadFlashBlock(comport : word; block : byte;ourbuffer : bufptr);
var
sstring : string;
retchar : char;
index : word;
begin
  sstring := SerReadFlashCom+ HexString(block,2)+CR;
  SendString(comport,sstring);
  for index := 1 to FlashBlockSize do
  begin
    HexByteRead(RecvString(comport,2),ourbuffer^[index]);
  end;
  PolledGetComChar(comport,retchar);
end;

function SerSync(comport : word) : boolean;
var
retchar : char;
sstring : string;
begin
  SerSync := false;
  ForceRTSHigh(comport);
  PolledPutComChar(ComPort,'%'); { send invalid command or data }
  PolledPutComChar(ComPort,CR);
  if PolledGetComCharWithTimeout(ComPort,retchar,1000000) <> 0 then
  begin
    TossChars(comport,20);
    PolledPutComChar(ComPort,'%'); { send invalid command or data }
    PolledPutComChar(ComPort,CR);
    if PolledGetComCharWithTimeout(ComPort,retchar,1000000) <> 0 then
    begin
      TossChars(comport,20);
      if retchar = '?' then
      begin
        sstring := SerWriteWordCom +'80000000' + CR;
        SendString(ComPort,sstring);
        if PolledGetComCharWithTimeout(ComPort,retchar,1000000) <> 0 then
        begin
          if retchar = CR then SerSync := true;
        end;
      end;
    end;
  end;
end;
