library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity CommandReg7 is
    Port ( 	
	 		clk: in STD_LOGIC;
			procibus: in STD_LOGIC_VECTOR (15 downto 0);
			procobus: out STD_LOGIC_VECTOR (15 downto 0);
			procreadcommand: in STD_LOGIC;
			procwritestatus: in STD_LOGIC;
			procreadstatus: in STD_LOGIC;
			hostibus: in STD_LOGIC_VECTOR (7 downto 0);
			hostobus: out STD_LOGIC_VECTOR (7 downto 0);
			hostadd: in STD_LOGIC;
			hostreadcommand: in STD_LOGIC;
			hostwritecommand: in STD_LOGIC;
			clrbusyflag: in STD_LOGIC;
			busyflag: out STD_LOGIC
			);
end CommandReg7;

architecture behavioral of CommandReg7 is

signal hostcommandreg: STD_LOGIC_VECTOR (15 downto 0);
signal procstatusreg: STD_LOGIC_VECTOR (15 downto 0);
signal busyf: STD_LOGIC;


begin	acommandreg: process
					(
						clk,
						hostcommandreg,
						procstatusreg,
						busyf,								
						procibus,
						procreadcommand,
						procwritestatus,
						procreadstatus,
						hostadd,
						hostibus,
						hostwritecommand,
						hostreadcommand,
						clrbusyflag						
						)
							
	begin
		if clk'event and clk = '1' then
			-- proc write logic

			if procwritestatus = '1' then
				procstatusreg(15 downto 0)  <= procibus;
			end if;

			-- host write logic

			if hostwritecommand = '1' then 
 		  		if hostadd = '0' then					
					hostcommandreg(7 downto 0) <= hostibus;
				else
					hostcommandreg(15 downto 8) <= hostibus;
					busyf <=  '1';
				end if;
			end if;

			if clrbusyflag = '1' then
 				busyf <= '0';
			end if;

		end if; 

		--	host read logic
			
		if	hostreadcommand = '1' then
			if hostadd = '0' then
				hostobus <= procstatusreg(7 downto 0);
			else
				hostobus(6 downto 0) <= procstatusreg(14 downto 8);
				hostobus(7) <= busyf;
			end if;
		else 
			hostobus <= "ZZZZZZZZ";
		end if;

		-- proc read logic
		
		procobus <= "ZZZZZZZZZZZZZZZZ";		
		if (procreadcommand or procreadstatus) = '1' then
			if procreadcommand = '1' and procreadstatus = '0' then
				procobus <= hostcommandreg;
			end if;
			if procreadstatus = '1' and procreadcommand = '0' then
				procobus <= procstatusreg;
			end if;
		else
			procobus <= "ZZZZZZZZZZZZZZZZ";
		end if;	
		busyflag <= busyf;
	end process;

end behavioral;
