library IEEE;
use IEEE.STD_LOGIC_1164.ALL;



entity DataReg7 is
    Port ( 	
	 		clk: in STD_LOGIC;
			procibus: in STD_LOGIC_VECTOR (15 downto 0);
			procobus: out STD_LOGIC_VECTOR (15 downto 0);
			procradd: in STD_LOGIC;
			procwadd: in STD_LOGIC;
			procreaddata: in STD_LOGIC;						
			procwritedata: in STD_LOGIC;
			hostibus: in STD_LOGIC_VECTOR (7 downto 0);
			hostobus: out STD_LOGIC_VECTOR (7 downto 0);
			hostadd: in STD_LOGIC_VECTOR (1 downto 0);
			hostreaddata: in STD_LOGIC;
			hostwritedata: in STD_LOGIC;
			hostdataclear: in STD_LOGIC
			);
end DataReg7;

architecture behavioral of DataReg7 is


signal hostdatareg: STD_LOGIC_VECTOR (31 downto 0);
signal procdatareg: STD_LOGIC_VECTOR (31 downto 0);


begin	adatareg: process
					(
						clk,
						hostdatareg,
						procdatareg,				
						procibus,
						procradd,
						procwadd,
						procwritedata,
						procreaddata,					
						hostibus,
						hostadd,
						hostwritedata,
						hostreaddata,
						hostdataclear
						)
							
	begin
		if clk'event and clk = '1' then
			-- proc write logic
  
			if procwritedata = '1' then
				case procwadd is
					when '0' => procdatareg(15 downto 0)  <= procibus;
					when '1' => procdatareg(31 downto 16) <= procibus;					
					when others => 
				end case;
			end if;

			-- host write logic

			if hostwritedata = '1' then 
 				case hostadd is
			    	when b"00" =>  hostdatareg(7 downto 0) <= hostibus;
		    		when b"01" =>  hostdatareg(15 downto 8) <= hostibus;
	  				when b"10" =>  hostdatareg(23 downto 16) <= hostibus;
					when b"11" =>  hostdatareg(31 downto 24) <= hostibus;
					when others => null;
				end case;
			end if;
			if hostdataclear = '1' then
				hostdatareg <= x"00000000";
			end if;
		end if; -- clk 

		--		host read logic

		if	hostreaddata = '1' then
			case hostadd is
		 		when b"00" => hostobus <= procdatareg(7 downto 0);
		 		when b"01" => hostobus <= procdatareg(15 downto 8);
		 		when b"10" => hostobus <= procdatareg(23 downto 16);
		 		when b"11" => hostobus <= procdatareg(31 downto 24);
				when others => null;
			end case;
		else 
			hostobus <= "ZZZZZZZZ"; 
		end if;
		
		-- proc read logic		
	
		if procreaddata = '1'then
			case procradd is
				when '0' => procobus <= hostdatareg(15 downto 0);
				when '1' => procobus <= hostdatareg(31 downto 16);				
				when others =>
			end case;
		else
			procobus <= "ZZZZZZZZZZZZZZZZ";
		end if;

	end process;

end behavioral;
